import datetime
import json
import logging
import re
import sys
from base64 import decodestring

import boto
import boto.rds
import boto.ec2.cloudwatch

sys.path.insert(0, '/motadata/motadata/motadata python engine/com/motadata/traceorg/python')

logging.basicConfig(filename='/motadata/motadata/log/motadata-python-engine.log',
                    format='%(asctime)s %(levelname)s %(message)s',
                    datefmt='%d-%b-%Y %I:%M:%S %p', level=logging.WARN)

logger = logging.getLogger('Amazon RDS Metric Collector')


def discover(context):
    result = {}

    try:
        if context.get('instance-id') is not None:
            is_running = is_rds_instance_running(context)
            if context.get('error-code') is not None:
                result['error-code'] = context['error-code']
            elif is_running == True:
                result['result'] = "1"
        else:
            rds_instances = list_rds_instances(context)

            if rds_instances is not None and len(rds_instances) >= 0:

                result['result'] = rds_instances

            else:
                context[
                    'error-code'] = "Failed to connect Amazon RDS service.Either no valid internet connection or " \
                                    "credentials. "

    except Exception as exception:
        context['error-code'] = exception.__class__.__name__

    return result


def is_rds_instance_running(context):

    is_running = False

    regions = boto.rds.regions()

    instance_name = context['instance-id']

    access_key = context['access-key']

    secret_key = context['secret-key']

    is_found = False

    is_connected = False

    for regionName in regions:

        client = boto.rds.connect_to_region(regionName.name, aws_access_key_id=access_key,
                                            aws_secret_access_key=secret_key)

        try:

            db_instances = client.get_all_dbinstances()

            if db_instances is not None and len(db_instances) >= 0:

                is_connected = True

                for i in range(len(db_instances)):

                    db_instance = str(db_instances[i])

                    if db_instance is not None and len(db_instance) > 0:

                        if db_instance.split(":")[1] == instance_name:
                            if "available" == str(db_instances[i].status).lower() or "backing-up" == str(db_instances[i].status).lower():
                                is_found = True
                                break


        except Exception as ignored:
            logger.debug("ignored exception")
            logger.debug(ignored)

    if is_connected == False :
        context['error-code'] = "Failed to connect Amazon RDS service.Either no valid internet connection or " \
                                "credentials."

    if is_connected == True and is_found == False :
        context['error-code'] = instance_name+" is not found or running."

    if is_connected == True and is_found == True:
        is_running = True


    return is_running

def collect(context):
    regions = boto.rds.regions()

    instance_name = context['instance-id']

    discoverd_region_name = context['region']

    metric_names = {}

    access_key = context['access-key']

    secret_key = context['secret-key']

    set_metric_name(metric_names)

    metrics = {}

    for regionName in regions:

        client = boto.rds.connect_to_region(regionName.name, aws_access_key_id=access_key,
                                            aws_secret_access_key=secret_key)

        cloud_watch = boto.ec2.cloudwatch.connect_to_region(regionName.name, aws_access_key_id=access_key,
                                                            aws_secret_access_key=secret_key)

        try:

            db_instances = client.get_all_dbinstances()

            if db_instances is not None and len(db_instances) >= 0:

                for i in range(len(db_instances)):

                    db_instance = str(db_instances[i])


                    if db_instance is not None and len(db_instance) > 0:


                        if db_instance.split(":")[1] == instance_name and regionName.name == discoverd_region_name:

                            if "available" == str(db_instances[i].status).lower():

                                metrics['Instance Name'] = str(instance_name)

                                if db_instances[i].instance_class is not None:
                                    metrics["Instance Type"] = str(db_instances[i].instance_class)

                                if db_instances[i].status is not None:
                                    metrics["Instance State"] = str(db_instances[i].status)

                                if db_instances[i].availability_zone is not None:
                                    metrics["Instance Availability Zone"] = str(
                                        db_instances[i].availability_zone)

                                if db_instances[i].endpoint[0] is not None:
                                    metrics["Source Host"] = str(db_instances[i].endpoint[0])

                                if db_instances[i].endpoint[1] is not None:

                                    counter_value = re.match("^[\+-]?[0-9]*[\.]?[0-9]+([eE][\+-]?[0-9]+)?$",
                                                             str(db_instances[i].endpoint[1]))
                                    if counter_value is not None:
                                        metrics["Source Port"] = long(db_instances[i].endpoint[1])

                                if db_instances[i].create_time is not None:
                                    metrics["Instance Launch Time"] = str(db_instances[i].create_time)

                                if db_instances[i].backup_retention_period is not None:

                                    counter_value = re.match("^[\+-]?[0-9]*[\.]?[0-9]+([eE][\+-]?[0-9]+)?$",
                                                             str(db_instances[i].backup_retention_period))
                                    if counter_value is not None:
                                        metrics["Backup Retention Time (Days)"] = long(
                                            db_instances[i].backup_retention_period)

                                if db_instances[i].engine is not None:
                                    metrics["DB Type"] = str(db_instances[i].engine)

                                if db_instances[i].engine_version is not None:
                                    metrics["DB Version"] = str(db_instances[i].engine_version)

                                if db_instances[i].auto_minor_version_upgrade is not None:
                                    metrics["Auto Upgrade"] = str(db_instances[i].auto_minor_version_upgrade)

                                if db_instances[i].auto_minor_version_upgrade is not None:

                                    if not db_instances[i].multi_az:
                                        metrics['Multi Availability Zone'] = "No"
                                    else:
                                        metrics['Multi Availability Zone'] = "Yes"

                                if db_instances[i].allocated_storage is not None:
                                    counter_value = re.match("^[\+-]?[0-9]*[\.]?[0-9]+([eE][\+-]?[0-9]+)?$",
                                                             str(db_instances[i].allocated_storage))
                                    if counter_value is not None:
                                        metrics["Allocated Storage"] = long(db_instances[i].allocated_storage)

                                if db_instances[i].preferred_backup_window is not None:
                                    metrics["Backup Window"] = str(db_instances[i].preferred_backup_window)

                                if db_instances[i].latest_restorable_time is not None:
                                    metrics["Latest Restorable Time"] = str(
                                        db_instances[i].latest_restorable_time)

                                if db_instances[i].preferred_maintenance_window is not None:
                                    metrics["Preferred Maintenance Window"] = str(db_instances[
                                                                                      i].preferred_maintenance_window)

                                get_rds_performance_metrics(cloud_watch, instance_name, metrics, metric_names)
                                return metrics

        except Exception as ignored:
            logger.debug("ignored exception")
            logger.debug(ignored)



def get_rds_performance_metrics(cloud_watch, instance_name, metrics, metric_names):
    end_date = datetime.datetime.utcnow()

    start_date = datetime.datetime.utcnow() - datetime.timedelta(seconds=50 * 6)  # 1 Minute

    stats = {"Average"}

    dimensions = {'DBInstanceIdentifier': instance_name}

    for key, metric in metric_names.items():
        data_points = cloud_watch.get_metric_statistics(
            60,  # Granularity
            start_date,
            end_date,  #
            key,  # Metric name
            'AWS/RDS',  # Namespace
            stats,
            dimensions
        )

        for data_point in data_points:
            if data_point[u'Average'] is not None:

                metric_value = re.match("^[\+-]?[0-9]*[\.]?[0-9]+([eE][\+-]?[0-9]+)?$",
                                        str(data_point[u'Average']))

                if metric_value is not None:
                    metrics[metric] = long(data_point[u'Average'])


    if 'Disk Reads/Sec' in metrics and 'Disk Writes/Sec' in metrics:
        metrics['Disk I/O Ops/sec'] = long(long(metrics['Disk Reads/Sec'])+long(metrics['Disk Writes/Sec']))

    if 'Disk Read Latency (Sec)' in metrics and 'Disk Write Latency (Sec)' in metrics:
        metrics['Disk I/O Latency (sec)'] = long(long(metrics['Disk Read Latency (Sec)'])+long(metrics['Disk Write Latency (Sec)']))

    if 'Disk Read (Bytes/Sec)' in metrics and 'Disk Write (Bytes/Sec)' in metrics:
        metrics['Disk I/O Bytes/sec'] = long(long(metrics['Disk Read (Bytes/Sec)'])+long(metrics['Disk Write (Bytes/Sec)']))

    if 'Network IN Traffic (Bytes/Sec)' in metrics and 'Network OUT Traffic (Bytes/Sec)' in metrics:
        metrics['Network Traffic (Bytes/sec)'] = long(long(metrics['Network IN Traffic (Bytes/Sec)'])+long(metrics['Network OUT Traffic (Bytes/Sec)']))

def set_metric_name(metric_names):

    metric_names["CPUCreditUsage"] = "CPU Credit Usage"

    metric_names["CPUCreditBalance"] = "CPU Credit Balance"

    metric_names["CPUUtilization"] = "CPU (%)"

    metric_names["FreeStorageSpace"] = "Free Storage (Bytes)"

    metric_names["DatabaseConnections"] = "Connections"

    metric_names["BinLogDiskUsage"] = "BinLog Disk Usage (Bytes)"

    metric_names["ReadIOPS"] = "Disk Reads/Sec"

    metric_names["WriteIOPS"] = "Disk Writes/Sec"

    metric_names["DiskQueueDepth"] = "Disk Queue Length"

    metric_names["ReadLatency"] = "Disk Read Latency (Sec)"

    metric_names["WriteLatency"] = "Disk Write Latency (Sec)"

    metric_names["ReadThroughput"] = "Disk Read (Bytes/Sec)"

    metric_names["WriteThroughput"] = "Disk Write (Bytes/Sec)"

    metric_names["SwapUsage"] = "Swap Usage (Bytes)"

    metric_names["ReplicaLag"] = "Replica Lag (Sec)"

    metric_names["FreeableMemory"] = "Free Memory (Bytes)"

    metric_names["BurstBalance"] = "Brust Balance (%)"

    metric_names["NetworkReceiveThroughput"] = "Network IN Traffic (Bytes/Sec)"

    metric_names["NetworkTransmitThroughput"] = "Network OUT Traffic (Bytes/Sec)"

    metric_names["OldestReplicationSlotLag"] = "Old Replication Slot Lag (MB)"

    metric_names["ReplicationSlotDiskUsage"] = "Replication Slot Disk Usage (MB)"

    metric_names["TransactionLogsDiskUsage"] = "Transaction Log Disk Usage (MB)"

    metric_names["TransactionLogsGeneration"] = "Transaction Log Throughput (MB/Sec)"


def list_rds_instances(context):
    regions = boto.rds.regions()

    result = []

    access_key = context['access-key']

    secret_key = context['secret-key']

    for region in regions:

        client = boto.rds.connect_to_region(region.name, aws_access_key_id=access_key,
                                            aws_secret_access_key=secret_key)

        try:
            db_instances = client.get_all_dbinstances()

            if db_instances is not None and len(db_instances) >= 0:

                for i in range(len(db_instances)):

                    db_instance = str(db_instances[i])

                    if db_instance is not None and len(db_instance) > 0:
                        result.append(str(db_instance.split(":")[1]).strip()+":"+str(region.name))

        except Exception as ignored:
            logger.debug("ignored exception")
            logger.debug(ignored)

    return result


if sys.argv[1] == 'collector':
    try:
        context = json.loads(decodestring(sys.argv[2]))

        result = collect(context)

        if result is not None:
            context['result'] = result

        del context['access-key']

        del context['secret-key']

    except Exception as exception:

        context['error-code1'] = exception.__class__.__name__
    finally:

        print json.dumps(context)

else:
    try:
        context = json.loads(decodestring(sys.argv[2]))

        result = {}

        result = discover(context)

        if result is not None and result.get('result') is not None:
            context['result'] = result['result']

        if context.get('error-code') is not None:
            context['error-code'] = context['error-code']

        del context['access-key']

        del context['secret-key']

    except Exception as exception:
        context['error-code1'] = exception.__class__.__name__
    finally:
        print json.dumps(context)